/********************************************************************************
** Form generated from reading UI file 'correctionwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CORRECTIONWIDGET_H
#define UI_CORRECTIONWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_CorrectionWidget
{
public:
    QVBoxLayout *verticalLayout;
    QFormLayout *formLayout;
    QLabel *givenKindLabel;
    QLabel *kindLabel;
    QLabel *givenIdentifierLabel;
    QLabel *identifierLabel;
    QLabel *typeLabel;
    QLineEdit *typeText;
    QLabel *importsLabel;
    QLineEdit *importsText;
    QDialogButtonBox *buttonBox;

    void setupUi(QWidget *CorrectionWidget)
    {
        if (CorrectionWidget->objectName().isEmpty())
            CorrectionWidget->setObjectName("CorrectionWidget");
        CorrectionWidget->resize(484, 167);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(CorrectionWidget->sizePolicy().hasHeightForWidth());
        CorrectionWidget->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(CorrectionWidget);
        verticalLayout->setObjectName("verticalLayout");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        givenKindLabel = new QLabel(CorrectionWidget);
        givenKindLabel->setObjectName("givenKindLabel");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, givenKindLabel);

        kindLabel = new QLabel(CorrectionWidget);
        kindLabel->setObjectName("kindLabel");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, kindLabel);

        givenIdentifierLabel = new QLabel(CorrectionWidget);
        givenIdentifierLabel->setObjectName("givenIdentifierLabel");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, givenIdentifierLabel);

        identifierLabel = new QLabel(CorrectionWidget);
        identifierLabel->setObjectName("identifierLabel");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, identifierLabel);

        typeLabel = new QLabel(CorrectionWidget);
        typeLabel->setObjectName("typeLabel");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, typeLabel);

        typeText = new QLineEdit(CorrectionWidget);
        typeText->setObjectName("typeText");

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, typeText);

        importsLabel = new QLabel(CorrectionWidget);
        importsLabel->setObjectName("importsLabel");

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, importsLabel);

        importsText = new QLineEdit(CorrectionWidget);
        importsText->setObjectName("importsText");

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, importsText);


        verticalLayout->addLayout(formLayout);

        buttonBox = new QDialogButtonBox(CorrectionWidget);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(CorrectionWidget);

        QMetaObject::connectSlotsByName(CorrectionWidget);
    } // setupUi

    void retranslateUi(QWidget *CorrectionWidget)
    {
        CorrectionWidget->setWindowTitle(tr2i18n("Specify correct type", nullptr));
        givenKindLabel->setText(tr2i18n("Kind:", nullptr));
        kindLabel->setText(QString());
        givenIdentifierLabel->setText(tr2i18n("Identifier:", nullptr));
        identifierLabel->setText(QString());
        typeLabel->setText(tr2i18n("Correct type:", nullptr));
        typeText->setPlaceholderText(tr2i18n("Code which evaluates to the type, e.g. int()", nullptr));
        importsLabel->setText(tr2i18n("Required modules:", nullptr));
        importsText->setPlaceholderText(tr2i18n("A comma separated list of modules required for the type", nullptr));
    } // retranslateUi

};

namespace Ui {
    class CorrectionWidget: public Ui_CorrectionWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CORRECTIONWIDGET_H

