// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "variabledebug.h"


Q_LOGGING_CATEGORY(KDEV_PYTHON_VARIABLECONTROLLER, "kdevelop.plugins.python.debugger.variablecontroller", QtInfoMsg)

